@@ -18,6 +18,7 @@ void xinitscr()
     raw();
     noecho();
     keypad(stdscr, TRUE);
+    //curs_set(0);
 
     int background = COLOR_BLACK;
 
@@ -97,6 +98,12 @@ void merr_msg(char* msg, ...)
             refresh();
 
             (void)getch();
+
+#if defined(__CYGWIN__)
+            xclear_immediately();       // 画面の再描写
+            view();
+            refresh();
+#endif
         }
         else {
             fprintf(stderr, "%s", msg2);
@@ -128,7 +135,7 @@ void msg_nonstop(char* msg, ...)
     const int maxy = mgetmaxy();
     const int maxx = mgetmaxx();
 
-    clear();
+    xclear();
     view();
     mclear_online(maxy-2);
     mclear_online(maxy-1);
@@ -191,7 +198,7 @@ char* choise(char* msg, char* str[], int len, int cancel)
             if(cursor < 0) cursor= 0;
         }
         else if(key == 12) {            // CTRL-L
-            mclear_immediately();
+            xclear_immediately();
         }
         else if(key == 3 || key == 7 || key == 27) { // CTRL-C -G Escape
             return NULL;
@@ -208,6 +215,12 @@ char* choise(char* msg, char* str[], int len, int cancel)
     }
 finished:
 
+#if defined(__CYGWIN__)
+    xclear_immediately();       // 画面の再描写
+    view();
+    refresh();
+#endif
+
     return str[cursor];
 }
 
@@ -257,8 +270,7 @@ int input_box(char* msg, char* result, int result_size, char* def_input, int def
     int result2 = 0;
     gInputBoxCursor = def_cursor;
 
-    if(gInputBoxInput) string_delete_malloc(gInputBoxInput);
-    gInputBoxInput = STRING_NEW_MALLOC(def_input);
+    string_put(gInputBoxInput, def_input);
     
     gView = input_box_view;
     
@@ -390,7 +402,7 @@ int input_box(char* msg, char* result, int result_size, char* def_input, int def
             break;
         }
         else if(key == 12) {            // CTRL-L
-            mclear_immediately();
+            xclear_immediately();
         }
         else {
             if(meta == 0 && !(key >= 0 && key <= 27)) {
@@ -412,6 +424,12 @@ int input_box(char* msg, char* result, int result_size, char* def_input, int def
 
     mmove_immediately(maxy -2, 0);
 
+#if defined(__CYGWIN__)
+    xclear_immediately();       // 画面の再描写
+    view();
+    refresh();
+#endif
+
     return result2;
 }
 
@@ -487,7 +505,7 @@ int select_str(char* msg, char* str[], int len, int cancel)
             if(gSelectStrCursor < 0) gSelectStrCursor= 0;
         }
         else if(key == 12) {            // CTRL-L
-            mclear_immediately();
+            xclear_immediately();
         }
         else if(key == 3 || key == 7 | key == 27) { // CTRL-C -G Escape
             gSelectStrCursor = cancel;
@@ -507,6 +525,12 @@ finished:
     
     gView = NULL;
 
+#if defined(__CYGWIN__)
+    xclear_immediately();       // 画面の再描写
+    view();
+    refresh();
+#endif
+
     return gSelectStrCursor;
 }
 
@@ -541,7 +565,7 @@ int select_str2(char* msg, char* str[], int len, int cancel)
             if(gSelectStrCursor < 0) gSelectStrCursor= 0;
         }
         else if(key == 12) {            // CTRL-L
-            mclear_immediately();
+            xclear_immediately();
         }
         else if(key == 3 || key == 7 || key == 27) { // CTRL-C -G Escape
             gSelectStrCursor = cancel;
@@ -561,5 +585,60 @@ finished:
     
     gView = NULL;
 
+#if defined(__CYGWIN__)
+    xclear_immediately();       // 画面の再描写
+    view();
+    refresh();
+#endif
+
     return gSelectStrCursor;
 }
+
+void xclear_immediately()
+{
+#if defined(__CYGWIN__)
+    int y;
+    for(y=0; y<mgetmaxy(); y++) {
+        mclear_online(y);
+    }
+    refresh();
+#else
+    mclear_immediately();
+#endif
+}
+
+void mclear_lastline()
+{
+    char space[1024];
+    int x;
+
+    const int maxx = mgetmaxx();
+    const int maxy = mgetmaxy();
+
+    for(x=0; x<maxx-1; x++) {
+        space[x] = ' ';
+    }
+    space[x] = 0;
+
+    attron(0);
+    mvprintw(maxy-1, 0, space);
+}
+
+void xclear()
+{
+#if defined(__CYGWIN__)
+    int y;
+    for(y=0; y<mgetmaxy()-1; y++) {
+        mclear_online(y);
+    }
+    mclear_lastline();
+#else
+    clear();
+#endif
+}
+
+void gui_init()
+{
+    gInputBoxInput = STRING_NEW_STACK("");
+}
+
