@@ -1,5 +1,4 @@
 #include "common.h"
-
 #include "config.h"
 
 #if defined(HAVE_CURSES_H)
@@ -33,19 +32,7 @@
 #ifdef __LINUX__
 
 #include <limits.h>
-/*
-#define __USE_POSIX
-#define __USE_XOPEN_EXTENDED
-#define __USE_POSIX199309
-#define __USE_UNIX98
-*/
 #include <signal.h>
-/*
-#undef __USE_POSIX
-#undef __USE_XOPEN_EXTENDED
-#undef __USE_POSIX199309
-#undef __USE_UNIX98
-*/
 #include <sys/wait.h>
 
 #else
@@ -55,22 +42,15 @@
 #include <sys/wait.h>
 
 #endif
-/*
-#if !defined(__DARWIN__)
-extern int kill (__pid_t __pid, int __sig) __THROW;
-extern int setenv(const char *name, const char *value, int overwrite);
-extern void srandom (unsigned int __seed) __THROW;
-#endif
-*/
 
 int gMainLoop = -1;
-void (*gView)() = NULL;          // 登録描写関数
+void (*gView)() = NULL;          // drawing function
 
-char gTempDir[PATH_MAX];         // 一時ディレクトリ
-char gHomeDir[PATH_MAX];         // ホームディレクトリ
+char gTempDir[PATH_MAX];         // temorary directory path
+char gHomeDir[PATH_MAX];         // home directory path
 
 ///////////////////////////////////////////////////
-// キーボード入力処理
+// input
 ///////////////////////////////////////////////////
 static void input(int meta, int key)
 {
@@ -80,215 +60,46 @@ static void input(int meta, int key)
         kill(getpid(), SIGSTOP);
         xinitscr();
     }
+    /// menu ///
     else if(gActiveMenu) {
-        /// メニュー ///
-        menu_input(gActiveMenu, meta, key);
+        menu_input(meta, key);
     }
-    /// インクリメンタルサーチ ///
+    /// incremental search ///
     else if(gISearch) {
         isearch_input(meta, key);
     }
+    /// filer ///
     else {
-        /// ファイラー ///
         filer_input(meta, key);
     }
 }
 
 ///////////////////////////////////////////////////
-// 描写
+// drawing
 ///////////////////////////////////////////////////
-static void job_view()
-{
-    const int maxy = mgetmaxy();
-    const int maxx = mgetmaxx();
-
-    const int size = xyzsh_job_num();
-    int i;
-    for(i=0; i<size; i++) {
-        char* title = xyzsh_job_title(i);
-
-        const int x = (maxx/size)*i;
-        char buf[BUFSIZ];
-        snprintf(buf, BUFSIZ, "[%d]%s", i+1, title);
-        char buf2[BUFSIZ];
-        
-        str_cut(gKanjiCode, buf, maxx/size, buf2, BUFSIZ);
-
-        mvprintw(maxy-3, x, "%s", buf2);
-    }
-}
-
-static void cmdline_view_filer()
-{
-    const int maxx = mgetmaxx();        // 画面サイズ
-    const int maxy = mgetmaxy();
-    
-    stack_start_stack();
-
-    sObject* fun = FUN_NEW_STACK(NULL);
-    sObject* stackframe = UOBJECT_NEW_GC(8, gxyzshObject, "_stackframe", FALSE);
-    vector_add(gStackFrames, stackframe);
-    //uobject_init(stackframe);
-    SFUN(fun).mLocalObjects = stackframe;
-
-    sObject* nextout = FD_NEW_STACK(kFDKindBuf, 0);
-
-    int rcode;
-    if(!run(gMFiler4Prompt, gStdin, nextout, &rcode, gCurrentObject, fun)) {
-        if(rcode == RCODE_BREAK) {
-            fprintf(stderr, "invalid break. Not in a loop\n");
-        }
-        else if(rcode == RCODE_RETURN) {
-            fprintf(stderr, "invalid return. Not in a function\n");
-        }
-        else if(rcode == RCODE_EXIT) {
-            fprintf(stderr, "invalid exit. In the prompt\n");
-        }
-        else {
-            fprintf(stderr, "run time error\n");
-            fprintf(stderr, "%s", string_c_str(gErrMsg));
-        }
-    }
-
-    mvprintw(maxy-2, 0, SFD(nextout).fdbuf.mBuf);
-
-    (void)vector_pop_back(gStackFrames);
-    stack_end_stack();
-
-    /// カーソル下のファイルの情報を描写 ///          
-    char buf[1024];
-    sFile* file = filer_cursor_file(adir());
-
-    char permission[12];
-    strncpy(permission, "----------", 12);
-
-    if(S_ISDIR(file->mLStat.st_mode)) permission[0] = 'd';  // ファイルの種別
-    if(S_ISCHR(file->mLStat.st_mode)) permission[0] = 'c';
-    if(S_ISBLK(file->mLStat.st_mode)) permission[0] = 'b';
-    if(S_ISFIFO(file->mLStat.st_mode)) permission[0] = 'p';
-    if(S_ISLNK(file->mLStat.st_mode)) permission[0] = 'l';
-    if(S_ISSOCK(file->mLStat.st_mode)) permission[0] = 's';
-
-    mode_t smode = file->mLStat.st_mode;        // パーミッション
-    if(smode & S_IRUSR) permission[1] = 'r';
-    if(smode & S_IWUSR) permission[2] = 'w';
-    if(smode & S_IXUSR) permission[3] = 'x';
-    if(smode & S_IRGRP) permission[4] = 'r';
-    if(smode & S_IWGRP) permission[5] = 'w';
-    if(smode & S_IXGRP) permission[6] = 'x';
-    if(smode & S_IROTH) permission[7] = 'r';
-    if(smode & S_IWOTH) permission[8] = 'w';
-    if(smode & S_IXOTH) permission[9] = 'x';
-    if(smode & S_ISUID) permission[3] = 's';
-    if(smode & S_ISGID) permission[6] = 's';
-#if defined(S_ISTXT)
-    if(smode & S_ISTXT) permission[9] = 't';
-#endif
-#if defined(S_ISVTX)
-    if(smode & S_ISVTX) permission[9] = 't';
-#endif
-    
-    permission[10] = 0;
-
-    time_t t = file->mLStat.st_mtime;                   // 時間
-    struct tm* tm_ = (struct tm*)localtime(&t);
-    
-    char buf2[1024];
-
-    char owner[256];
-    char* tmp = string_c_str(file->mUser);
-    if(tmp) {
-        str_cut2(gKanjiCode, tmp, 8, owner, 256);
-    }
-    else {
-        snprintf(owner, 256, "%d", file->mLStat.st_uid);
-    }
-
-    char group[256];
-    tmp = string_c_str(file->mGroup);
-    if(tmp) {
-        str_cut2(gKanjiCode, tmp, 7, group, 256);
-    }
-    else {
-        snprintf(group, 256, "%d", file->mLStat.st_gid);
-    }
-
-    char size_buf[256];
-    int width = filer_make_size_str(size_buf, file->mLStat.st_size);
-
-    if(S_ISDIR(file->mStat.st_mode)) {
-        strncpy(size_buf, "<DIR>", 256);
-    }
-
-    char format[128];
-    strncpy(format, "%", 128);
-    snprintf(format + 1, 126, "%d", width);
-    strncat(format, "s", 128);
-
-    char size_buf2[256];
-    snprintf(size_buf2, 256, format, size_buf);
-    if(!S_ISLNK(file->mLStat.st_mode)) {
-        int year = tm_->tm_year-100;
-        if(year < 0) year+=100;
-        while(year > 100) year-=100;
-        
-        snprintf(buf2, 1024, 
-            "%s %3d %-8s %-7s%s %02d-%02d-%02d %02d:%02d %s"
-            //, "%s %3d %-8s %-7s%10lld %02d-%02d-%02d %02d:%02d %s"
-            , permission, file->mLStat.st_nlink
-            , owner, group
-                                
-            , size_buf2
-            
-            , year, tm_->tm_mon+1, tm_->tm_mday
-            , tm_->tm_hour, tm_->tm_min
-            , string_c_str(file->mNameView));
-            
-        char buf3[1024];
-        str_cut2(gKanjiCode, buf2, maxx-1, buf3, 1024);
-                            
-        mvprintw(maxy-1, 0, "%s", buf3);
-    }
-    else {
-        snprintf(buf, 1024,
-           "%s %3d %s%s%s %02d-%02d-%02d %02d:%02d %s -> %s"
-           //, "%s %3d %s%s%10lld %02d-%02d-%02d %02d:%02d %s -> %s"
-           , permission, file->mLStat.st_nlink
-           , owner, group
-                                
-           , size_buf2
-
-           , tm_->tm_year-100, tm_->tm_mon+1, tm_->tm_mday
-           , tm_->tm_hour, tm_->tm_min
-           , string_c_str(file->mNameView)
-           , string_c_str(file->mLinkTo));
-           
-        char buf2[1024];
-        str_cut2(gKanjiCode, buf, maxx-1, buf2, 1024);
-                   
-        mvprintw(maxy-1, 0, "%s", buf2);
-    }
-
-    move(maxy-1, maxx-2);
-}
-
 void view()
 {
+    /// incremental search ///
     if(gISearch) {
         filer_view(0);
         filer_view(1);
         job_view();
 
         isearch_view();
+#if defined(__CYGWIN__)
+        cmdline_view_filer();
+#endif
     }
+    /// menu ///
     else if(gActiveMenu) {
         filer_view(0);
         filer_view(1);
         job_view();
 
-        menu_view(gActiveMenu);
+        menu_view();
         cmdline_view_filer();
     }
+    /// filer ///
     else {
         filer_view(0);
         filer_view(1);
@@ -300,7 +111,7 @@ void view()
 }
 
 ///////////////////////////////////////////////////
-// ランタイムスクリプト実行
+// runtime script
 ///////////////////////////////////////////////////
 static BOOL read_rc_file(int argc, char** argv)
 {
@@ -329,19 +140,19 @@ static BOOL read_rc_file(int argc, char** argv)
 }
 
 ///////////////////////////////////////////////////
-// exit時に実行される関数
+// on exiting
 ///////////////////////////////////////////////////
 static void atexit_fun()
 {
 }
 
 ///////////////////////////////////////////////////
-// シグナル処理
+// signal
 ///////////////////////////////////////////////////
 static void sig_winch(int s)
 {
     if(gMainLoop == -1) {
-        mclear_immediately();       // 画面の再描写
+        xclear_immediately();       // refresh
         view();
         refresh();
     }
@@ -350,13 +161,12 @@ static void sig_winch(int s)
 static void sig_cont(int s)
 {
     if(gMainLoop == -1) {
-        mclear_immediately();       // 画面の再描写
+        xclear_immediately();       // refresh
         view();
         refresh();
     }
 }
 
-
 void set_signal_mfiler()
 {
     xyzsh_set_signal();
@@ -381,7 +191,7 @@ void set_signal_mfiler()
 }
 
 ///////////////////////////////////////////////////
-// メイン関数
+// main function
 ///////////////////////////////////////////////////
 static void usage()
 {
@@ -751,7 +561,7 @@ static void set_mfenv()
     setenv("ma_red", buf, 1);
 }
 
-static void mygetcwd(char* result, int result_size)
+void mygetcwd(char* result, int result_size)
 {
     int l;
     
@@ -770,148 +580,31 @@ void extname(char* result, int result_size, char* name)
     
     for(p = name + strlen(name); p != name; p--) {
         if(*p == '.' && p == name + strlen(name)) {
-            strncpy(result, "", result_size);
+            xstrncpy(result, "", result_size);
             return;
         }
         else if(*p == '/') {
-            strncpy(result, "", result_size);
+            xstrncpy(result, "", result_size);
             return;
         }
         else if(*p == '.') {
-            strncpy(result, p+1, result_size);
+            xstrncpy(result, p+1, result_size);
             return;
         }
     }
 
-    strncpy(result, "", result_size);
-}
-
-int correct_path(char* current_path, char* path, char* path2, int path2_size)
-{
-    char tmp[PATH_MAX];
-    char tmp2[PATH_MAX];
-    char tmp3[PATH_MAX];
-    char tmp4[PATH_MAX];
-
-    /// 先頭にカレントパスを加える ///
-    {
-        if(path[0] == '/') {      /// 絶対パスなら必要ない
-            strncpy(tmp, path, PATH_MAX);
-        }
-        else {
-            if(current_path == NULL) {
-                char cwd[PATH_MAX];
-                mygetcwd(cwd, PATH_MAX);
-                
-                strncpy(tmp, cwd, PATH_MAX);
-                strncat(tmp, "/", PATH_MAX);
-                strncat(tmp, path, PATH_MAX);
-            }
-            else {
-                strncpy(tmp, current_path, PATH_MAX);
-                if(current_path[strlen(current_path)-1] != '/') {
-                    strncat(tmp, "/", PATH_MAX);
-                }
-                strncat(tmp, path, PATH_MAX);
-            }
-        }
-
-    }
-
-    strncpy(tmp2, tmp, PATH_MAX);
-
-    /// .を削除する ///
-    {
-        char* p;
-        char* p2;
-        int i;
-
-        p = tmp3;
-        p2 = tmp2;
-        while(*p2) {
-            if(*p2 == '/' && *(p2+1) == '.' && *(p2+2) != '.' && ((*p2+3)=='/' || *(p2+3)==0)) {
-                p2+=2;
-            }
-            else {
-                *p++ = *p2++;
-            }
-        }
-        *p = 0;
-
-        if(*tmp3 == 0) {
-            *tmp3 = '/';
-            *(tmp3+1) = 0;
-        }
-
-    }
-
-    /// ..を削除する ///
-    {
-        char* p;
-        char* p2;
-
-        p = tmp4;
-        p2 = tmp3;
-
-        while(*p2) {
-            if(*p2 == '/' && *(p2+1) == '.' && *(p2+2) == '.' 
-                && *(p2+3) == '/')
-            {
-                p2 += 3;
-
-                do {
-                    p--;
-
-                    if(p < tmp4) {
-                        strncpy(path2, "/", path2_size);
-                        return FALSE;
-                    }
-                } while(*p != '/');
-
-                *p = 0;
-            }
-            else if(*p2 == '/' && *(p2+1) == '.' && *(p2+2) == '.' 
-                && *(p2+3) == 0) 
-            {
-                do {
-                    p--;
-
-                    if(p < tmp4) {
-                        strncpy(path2, "/", path2_size);
-                        return FALSE;
-                    }
-                } while(*p != '/');
-
-                *p = 0;
-                break;
-            }
-            else {
-                *p++ = *p2++;
-            }
-        }
-        *p = 0;
-    }
-
-    if(*tmp4 == 0) {
-        strncpy(path2, "/", path2_size);
-    }
-    else {
-        strncpy(path2, tmp4, path2_size);
-    }
-
-    return TRUE;
+    xstrncpy(result, "", result_size);
 }
 
 int main(int argc, char* argv[])
 {
-    /// メモリリークの監視を開始 ///
-    CHECKML_BEGIN(FALSE);
+    CHECKML_BEGIN(FALSE);    // start to watch memory leak
 
-    /// ファイラでもスクリプトでも共通の環境変数を初期化 ///
-    setenv("VERSION", "1.0.5", 1);
+    /// initialization for envronment variable ///
+    setenv("VERSION", "1.0.8", 1);
     setenv("MFILER4_DATAROOTDIR", DATAROOTDIR, 1);
 
-    /// mfiler4のホームディレクトリを保存しておく ///
+    /// save mfiler4 home directory ///
     char* home = getenv("HOME");
     if(home == NULL) {
         fprintf(stderr, "$HOME is NULL.exit");
@@ -933,7 +626,7 @@ int main(int argc, char* argv[])
         exit(1);
     }
 
-    /// mfiler4の一時ディレクトリを保存しておく ///
+    /// save mfiler4 temporary directory ///
     snprintf(gTempDir, PATH_MAX, "%s/temp", gHomeDir);
     if(access(gTempDir, F_OK) != 0) {
         if(mkdir(gTempDir, 0700) < 0) {
@@ -947,28 +640,24 @@ int main(int argc, char* argv[])
         exit(1);
     }
 
-    /// 乱数初期化 ///
     srandom(1000);
 
-    /// 端末の設定を保存 ///
     msave_ttysettings();
-
-    /// 端末を初期化 ///
     mreset_tty();
 
-    /// オプション処理 ///
-    char send_cmd[BUFSIZ];          // コマンドがセットされていたら
-    char file_name[PATH_MAX];       // ファイル名が引数の場合
+    /// optoin
+    char send_cmd[BUFSIZ];
+    char file_name[PATH_MAX];
     file_name[0] = 0;
 
     int i;
     int pid;
     for(i=1 ; i<argc; i++){
-        // ファイル名
-        if(argv[i][0] != '-') {         // オプションじゃなければ
-            xstrncpy(file_name, argv[i], PATH_MAX);     // ファイル名とみなす
+        // file name
+        if(argv[i][0] != '-') {
+            xstrncpy(file_name, argv[i], PATH_MAX);
         }
-        // 引数
+        // argument
         else {
             if(strcmp(argv[i], "--version") == 0) version();
 
@@ -989,31 +678,32 @@ int main(int argc, char* argv[])
         }
     }
 
-    /// 標準入力と出力が制御端末かどうか確認 ///
     if(!isatty(0) || !isatty(1)) {
         fprintf(stderr, "standard input is not a tty\n");
         return 1;
     }
 
-    /// 環境変数設定 ///
+    /// set environment variables ///
     set_mfenv();
 
-    /// モジュールの初期化 ///
+    /// initialization for all modules ///
     xyzsh_init(kATConsoleApp, FALSE);
-    menu_init();
-    filer_init();
     commands_init();
+    filer_init();
+    menu_init();
+    gui_init();
+    isearch_init();
 
-    /// シグナルを設定 ///
+    /// signal ///
     set_signal_mfiler();
 
-    /// localeの設定 ///
+    /// locale ///
     //if(gTerminalKanjiCode == kTKUtf8) setlocale(LC_CTYPE, "ja_JP.UTF-8");
 
-    /// atexit登録 ///
+    /// atexit ///
     atexit(atexit_fun);
 
-    /// ディレクトリを作成 ///
+    /// make directories ///
     if(vector_count(gDirs) == 0) {
         if(strcmp(file_name, "") == 0) {
             char cwd[PATH_MAX];
@@ -1043,15 +733,11 @@ int main(int argc, char* argv[])
 
     (void)filer_activate(0);
 
-    /// モジュールの初期化(ランタイムスクリプト以降の方が良い奴 ///
-    isearch_init();
-
-    /// 初期キーバインドと初期フック ///
-    if(!filer_add_keycommand2(0, 'q', "quit", "system", FALSE)) {
-        /// ジョブの消去 ///
+    /// keybind ///
+    if(!filer_add_keycommand2(0, 'q', "quit", "system", FALSE) || !filer_add_keycommand2(0, 3, "quit", "system", FALSE)) 
+    {
         xyzsh_kill_all_jobs();
 
-        /// モジュール解放 ///
         isearch_final();
         menu_final();
         filer_final();
@@ -1061,47 +747,20 @@ int main(int argc, char* argv[])
         mreset_tty();
         //mrestore_ttysettings();
 
-        /// メモリリーク検出 ///
         CHECKML_END();
 
-        /// 改行 ///
-        puts("");
-
-        mreset_tty();
-        exit(1);
-    }
-    if(!filer_add_keycommand2(0, 3, "quit", "system", FALSE)) {
-        /// ジョブの消去 ///
-        xyzsh_kill_all_jobs();
-
-        /// モジュール解放 ///
-        isearch_final();
-        menu_final();
-        filer_final();
-        xyzsh_final();
-        commands_final();
-
-        mreset_tty();
-        //mrestore_ttysettings();
-
-        /// メモリリーク検出 ///
-        CHECKML_END();
-
-        /// 改行 ///
         puts("");
 
         mreset_tty();
         exit(1);
     }
 
-    /// ランタイムスクリプト実行 ///
+    /// run time script ///
     gMainLoop = -1;
 
     if(!read_rc_file(argc, argv)) {
-        /// ジョブの消去 ///
         xyzsh_kill_all_jobs();
 
-        /// モジュール解放 ///
         isearch_final();
         menu_final();
         filer_final();
@@ -1111,20 +770,18 @@ int main(int argc, char* argv[])
         mreset_tty();
         //mrestore_ttysettings();
 
-        /// メモリリーク検出 ///
         CHECKML_END();
 
-        /// 改行 ///
         puts("");
 
         mreset_tty();
         exit(1);
     }
     
-    /// カーシスの起動 ///
+    /// start curses ///
     xinitscr();
 
-    /// メインループ ///
+    /// main loop ///
     fd_set mask;
     fd_set read_ok;
 
@@ -1132,15 +789,15 @@ int main(int argc, char* argv[])
     FD_SET(0, &mask);
 
     while(gMainLoop == -1) {
-        /// 描写 ///
-        clear();
+        /// drawing ///
+        xclear();
         view();
         refresh();
 
         const int maxy = mgetmaxy();
         mmove_immediately(maxy-2, 0);
 
-        /// selectで入力待ち ///
+        /// select for waiting to input ///
         read_ok = mask;
         if(xyzsh_job_num() > 0) {
             struct timeval tv;
@@ -1153,7 +810,7 @@ int main(int argc, char* argv[])
             select(5, &read_ok, NULL, NULL, NULL);
         }
 
-        /// キー処理 ///
+        /// input ///
         if(FD_ISSET(0, &read_ok)) {
             int meta;
             int key = xgetch(&meta);
@@ -1161,17 +818,17 @@ int main(int argc, char* argv[])
             input(meta, key);
         }
 
-        /// ジョブ処理 ///
+        /// works to background jobs ///
         xyzsh_wait_background_job();
     }
 
-    /// カーシス終わり ///
+    /// end curses ///
     endwin();
 
-    /// ジョブの消去 ///
+    /// kill all jobs ///
     xyzsh_kill_all_jobs();
 
-    /// モジュール解放 ///
+    /// finalization for modules ///
     isearch_final();
     menu_final();
     filer_final();
@@ -1181,10 +838,10 @@ int main(int argc, char* argv[])
     mreset_tty();
     //mrestore_ttysettings();
 
-    /// メモリリーク検出 ///
+    /// checking memory leaks ///
     CHECKML_END();
 
-    /// 改行 ///
+    /// puts ///
     puts("");
 
     mreset_tty();
